/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import nl.cloudfarming.client.field.model.FieldImportSource;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.FilePickerUtil;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra
 */
@ActionID(category = "Project",
id = "nl.cloudfarming.client.field.ImportAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/field/map_add.png",
displayName = "#CTL_ImportFieldFiles")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 0)
})
@NbBundle.Messages({"CTL_ImportFieldFiles=Import Fields",
    "fileChooserTitle=Import field files",
    "fileChooserApproveText=Import",
    "Invalid import file message=The selected file {0} did not contain a valid import format",
    "importaction.filepicker.filefilter.extension=.shp",
    "importaction.filepicker.filefilter.description=Shapefile (*{0})",
    "importaction.filepicker.title=Shapefile Import",
    "importaction.filepicker.approve=Import",
    "importaction.error.import.not_a_shape_file=The selected file could NOT be recognized as a shapefile!",
    "importaction.action.import=Import Isobus"
})
public class ImportAction extends AbstractAction implements ActionListener {

    private final FieldProject project;
    private static final AppLogger APP_LOGGER = AppLogFactory.getLogger(ImportAction.class);
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.ImportAction");
    
    private static final String FILEFILTEREXTENSION = NbBundle.getMessage(ImportAction.class, "importaction.filepicker.filefilter.extension");
    private static final String FILEFILTER_DESCRIPTION = NbBundle.getMessage(ImportAction.class, "importaction.filepicker.filefilter.description", FILEFILTEREXTENSION);
    private static final String FILEPICKER_TITLE = NbBundle.getMessage(ImportAction.class, "importaction.filepicker.title");
    private static final String FILEPICKER_APPROVE = NbBundle.getMessage(ImportAction.class, "importaction.filepicker.approve");
    private static final String IMPORT_ERROR = NbBundle.getMessage(ImportAction.class, "importaction.error.import.not_a_shape_file");    
    
    public ImportAction(FieldProject project) {
        this.project = project;
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        
        // Build the filechooser
        FilePickerUtil.AgroSenseFilePicker filePicker = new FilePickerUtil.AgroSenseFilePicker("ImportAction-directory");
        filePicker.setTitle(FILEPICKER_TITLE);
        //agroSenseFilePicker.setDefaultWorkingDirectory(new File(IsobusPreference.LAST_DIR.getValue()));
        filePicker.setApproveText(FILEPICKER_APPROVE);
        filePicker.addFileFilter(FILEFILTER_DESCRIPTION, FILEFILTEREXTENSION);

        // Old chooser
//        List<String> supportedExtensions = new ArrayList<>();
//        supportedExtensions.add(".shp");
//        FileFilter ff = FilePickerUtil.createSwingFileFilter(supportedExtensions, "Shape file (*.shp)");
//        FileChooserBuilder filePicker = new FileChooserBuilder(ImportAction.class).setTitle(NbBundle.getMessage(this.getClass(), "fileChooserTitle")).setApproveText(NbBundle.getMessage(this.getClass(), "fileChooserApproveText")).addFileFilter(ff);

        File chosenFile = filePicker.showOpenDialog();

        if (chosenFile != null) {
            FileObject selectedFileObject = FileUtil.toFileObject(chosenFile);
            try {
                DataObject dataObject = DataObject.find(selectedFileObject);
                if (dataObject != null && dataObject.getLookup() != null && dataObject.getLookup().lookup(FieldImportSource.class) != null) {
                    for (FileObject fileObject : dataObject.getLookup().lookup(FieldImportSource.class).getFiles()) {
                        FileObject importFolder = project.getImportFolder(true);
                        String freeFileName = FileUtil.findFreeFileName(importFolder, fileObject.getName(), fileObject.getExt());
                        
                        FileUtil.copyFile(fileObject, importFolder, freeFileName);
                    }
                }else{
                    JOptionPane.showMessageDialog(null, IMPORT_ERROR);
                    APP_LOGGER.warning(Bundle.Invalid_import_file_message(chosenFile.getName()));
                }
            } catch (DataObjectNotFoundException ex) {
                Exceptions.printStackTrace(ex);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
    }
}